# Generated by Django 5.2.1 on 2026-02-02 17:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0010_alter_memberpreferences_member"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="memberpreferences",
            name="member",
        ),
        migrations.AddField(
            model_name="member",
            name="preferences",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="member",
                to="ichec_django_core.memberpreferences",
            ),
        ),
    ]
