# Generated by Django 5.2.1 on 2026-02-01 12:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0008_organization_active_organization_deleted_at_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="organization",
            options={
                "verbose_name": "Organization",
                "verbose_name_plural": "Organizations",
            },
        ),
    ]
