# Generated by Django 5.2.1 on 2026-01-30 17:21

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0007_member_active_member_deleted_at_member_public_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="organization",
            name="deleted_at",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AddField(
            model_name="organization",
            name="public",
            field=models.BooleanField(default=False),
        ),
    ]
