# Generated by Django 5.2.1 on 2026-01-30 17:14

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0006_alter_memberpreferences_notifications"),
    ]

    operations = [
        migrations.AddField(
            model_name="member",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="member",
            name="deleted_at",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AddField(
            model_name="member",
            name="public",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="member",
            name="verified",
            field=models.BooleanField(
                default=False,
                help_text="Used to indicate that a user is regarded as 'verified' - for example  by verifying their email address. This allows the concept of  users who have been invited to a portal - but can't yet interact  with it in most ways. A user coming from an OIDC provider might be assumed verified - based on properties of that provider.",
            ),
        ),
    ]
