# Generated by Django 5.2.1 on 2026-01-20 15:59

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0004_alter_member_profile_alter_member_profile_thumbnail",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="email",
            field=models.CharField(max_length=200, null=True),
        ),
        migrations.CreateModel(
            name="MemberPreferences",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("notifications", models.JSONField(blank=True)),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Notification",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("message", models.CharField(max_length=1000)),
                ("link", models.CharField(max_length=100)),
                ("read_time", models.DateTimeField(null=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
