# Generated by Django 5.2.1 on 2026-01-20 16:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0005_organization_email_memberpreferences_notification"),
    ]

    operations = [
        migrations.AlterField(
            model_name="memberpreferences",
            name="notifications",
            field=models.JSONField(null=True),
        ),
    ]
