# Generated by Django 5.2.10 on 2026-02-02 08:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0009_alter_organization_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="memberpreferences",
            name="member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="preferences",
                to="ichec_django_core.member",
            ),
        ),
    ]
